function d_array = signal_constellation(DataType, n, OneTrainBlock, OneDataBlock)
% 'BPSK','QPSK'
% 'DBPSK','DQPSK'
% 'pi2DBPSK','pi4DQPSK','Delayed_DBPSK','Delayed_DQPSK'
% 'AsymBPSK','AsymQPSK','AsymDBPSK','AsymDQPSK'
% 'AsymBPSK_BPSK','AsymQPSK_QPSK','AsymDBPSK_DBPSK','AsymDQPSK_DQPSK'
% 'BPSK_AsymBPSK_End','QPSK_AsymQPSK_End','DBPSK_AsymDBPSK_End','DQPSK_AsymDQPSK_End'
% '64MRDPSK','64MRDPSK_DQPSK_AsymDQPSK_End'
% 'DQPSK_64MRDPSK_AsymDQPSK','DQPSK_64MRDPSK_AsymDQPSK_End','DQPSK[2]_64MRDPSK_AsymDQPSK_End'
% 'DBPSK_64MRDPSK_AsymDBPSK','DBPSK_64MRDPSK_AsymDBPSK_End','DBPSK[2]_64MRDPSK_AsymDBPSK_End'
% '8PSK','8DPSK','DQPSK_8DPSK_AsymDQPSK_End','DBPSK_8DPSK_AsymDBPSK_End'
% 'DQPSK[2]_8DPSK_AsymDQPSK_End','DBPSK[2]_8DPSK_AsymDBPSK_End'
% '16PSK','16DPSK','DQPSK_16DPSK_AsymDQPSK_End','DBPSK_16DPSK_AsymDBPSK_End'
% 'DQPSK[2]_16DPSK_AsymDQPSK_End','DBPSK[2]_16DPSK_AsymDBPSK_End'
% '16QAM','DQPSK_16QAM_AsymDQPSK_End','DBPSK_16QAM_AsymDBPSK_End',
% 'DQPSK[2]_16QAM_AsymDQPSK_End','DBPSK[2]_16QAM_AsymDBPSK_End'
% '64QAM','DQPSK_64QAM_AsymDQPSK_End','DBPSK_64QAM_AsymDQPSK_End',
% 'DQPSK[2]_64QAM_AsymDQPSK_End','DBPSK[2]_64QAM_ASymDBPSK_End'

% The ASC's are hard-coded.  This function generates all the possible values for different signal constellations.

if (nargin == 3)
   error('*** SIGNAL_CONSTELLATION.M : Missing Parameter DataBlock! ***');
end;
if (nargin == 4)
   % Only 1 row of TrainBlock and DataBlock is passed as parameter at a time.
   if ((size(OneTrainBlock,1) > 1) | (size(OneDataBlock,1) > 1))
      error('*** SIGNAL_CONSTELLATION.M : Invalid OneTrainBlock or OneDataBlock size! ***');
   else
      if ((length(OneTrainBlock)==1) | (length(OneDataBlock)==1))
         error('*** SIGNAL_CONSTELLATION.M : Invalid OneTrainBlock or OneDataBlock! ***');
      end;
      TB = [OneTrainBlock(1), OneTrainBlock(length(OneTrainBlock))];
      DB = [OneDataBlock(1), OneDataBlock(length(OneDataBlock))];
   end;
end;

switch (DataType)
case {'BPSK','DBPSK','Delayed_DBPSK'}
   d_array = [+1, -1];
case {'QPSK','DQPSK','Delayed_DQPSK'}
   d_array = [+1, +j, -1, -j];
case {'pi2DBPSK'}
   if ((nargin == 2) | (TB(1) == 0)) % or '0 Symbol'
      new_n = n;
   else
      new_n = n-TB(1)+1;
   end;
   if ((new_n/2) == ceil(new_n/2)) % Even
      d_array = signal_constellation('BPSK');
   else % Odd
      d_array = [+j, -j];
   end;
case {'pi4DQPSK'}
   if ((nargin == 2) | (TB(1) == 0)) % or '0 Symbol'
      new_n = n;
   else
      new_n = n-TB(1)+1;
   end;
   if ((new_n/2) == ceil(new_n/2)) % Even
      d_array = signal_constellation('QPSK');
   else % Odd
      d_array = exp([1, 3, 5, 7]/8*2*pi*j);
   end;
case {'AsymBPSK','AsymDBPSK'}
   d_array = [+1, +j];
case {'QAsymBPSK','QAsymDBPSK'}
   d_array = exp([0, 6]/16*2*pi*j);
case {'RevAsymBPSK', 'RevAsymDBPSK'}
   d_array = [+1, -j];
case {'RevQAsymBPSK', 'RevQAsymDBPSK'}
   d_array = exp([0, 11]/16*2*pi*j);   
case {'AsymQPSK','AsymDQPSK'}
   % Hard codes the points where the constellation is asymmetric and each point has the maximum separation between them.
   d_array = exp([0, 3, 6, 9]/16*2*pi*j);
case {'AsymBPSK_BPSK','AsymDBPSK_DBPSK'}
   %%% Train[Asym(D)BPSK] Data[(D)BPSK]
   if ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymBPSK');
   elseif ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('BPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
   
case {'AsymBPSK_QPSK','AsymDBPSK_DQPSK'}
%    n
%[TB(1) TB(2)]
%[DB(1) DB(2)]
   %%% Train[Asym(D)BPSK] Data[(D)BPSK]
   if ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('QAsymBPSK');
   elseif ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('QPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
   
   
case {'AsymQPSK_QPSK','AsymDQPSK_DQPSK'}
   %%% Train[Asym(D)QPSK] Data[(D)QPSK]
   if ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymQPSK');
   elseif ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('QPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'BPSK_AsymBPSK_End','DBPSK_AsymDBPSK_End'}
   %%% Data[(D)BPSK] Train[Asym(D)BPSK]
   if ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('BPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
   
   
case {'QPSK_AsymQPSK_End','DQPSK_AsymDQPSK_End', 'DQPSK_AsymDQPSK_DQPSK_End'}
   %%% Data[(D)QPSK] Train[Asym(D)QPSK]
   if ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('QPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;

   
   
case {'DQPSK_AsymDBPSK[2]_DQPSK_End'}
%    DB(1)
%    DB(2)
%    TB(1)
%    TB(2)
%    n
    
   if ((DB(1) <= n) & (n <= DB(2)))
      d_array = signal_constellation('QPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('QAsymBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
   
  
   
   
% 'DQPSK_64MRDPSK_AsymDQPSK','DQPSK_64MRDPSK_AsymDQPSK_End'
case {'64MRDPSK'}
   delta = 0.49;
   delta_0 = 0.5148^2;
   xi = sqrt(delta_0+([0:3]*delta));
   dxi = 2*pi/16;
   d_array = xi;
   for i = 1:15;
      d_array = [d_array, xi*exp(j*dxi*i)];
   end;
case {'64MRDPSK_DQPSK_AsymDQPSK_End'}
   %%% Data[64MRDPSK,DQPSK] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(2)-1))
      d_array = signal_constellation('64MRDPSK');
   elseif (n == DB(2))
      d_array = signal_constellation('DQPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK_64MRDPSK_AsymDQPSK'}
   %%% Train[DQPSK] Data[64MRDPSK,AsymDQPSK]
   if ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1) <= n) & (n <= DB(2)-1))
      d_array = signal_constellation('64MRDPSK');
   elseif (n == DB(2))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK_64MRDPSK_AsymDQPSK_End'}
   %%% Data[DQPSK,64MRDPSK] Train[AsymDQPSK]
   if (n == DB(1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64MRDPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK[2]_64MRDPSK_AsymDQPSK_End'}
   %%% Data[DQPSK[2],64MRDPSK] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64MRDPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_64MRDPSK_AsymDBPSK'}
   %%% Train[DBPSK] Data[64MRDPSK,AsymDBPSK]
   if ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1) <= n) & (n <= DB(2)-1))
      d_array = signal_constellation('64MRDPSK');
   elseif (n == DB(2))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_64MRDPSK_AsymDBPSK_End'}
   %%% Data[DBPSK,64MRDPSK] Train[AsymDBPSK]
   if (n == DB(1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64MRDPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK[2]_64MRDPSK_AsymDBPSK_End'}
   %%% Data[DBPSK[2],64MRDPSK] Train[AsymDBPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64MRDPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'8PSK','8DPSK'}
   d_array = exp([0:7]/8*2*pi*j);
case {'DQPSK_8DPSK_AsymDQPSK_End'}
   %%% Data[DQPSK,8DPSK] Train[AsymDQPSK]
   if (n == DB(1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('8DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_8DPSK_AsymDBPSK_End'}
   %%% Data[DBPSK,8DPSK] Train[AsymDBPSK]
   if (n == DB(1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('8DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
        
case {'DQPSK[2]_8DPSK_AsymDQPSK_End'}
   %%% Data[DQPSK[2],8DPSK] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('8DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK[2]_8DPSK_AsymDBPSK_End'}
   %%% Data[DBPSK[2],8DPSK] Train[AsymDBPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('8DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'16PSK','16DPSK'}
   d_array = exp([0:15]/16*2*pi*j);
case {'DQPSK_16DPSK_AsymDQPSK_End'}
   %%% Data[DQPSK,16DPSK] Train[AsymDQPSK]
   if (n == DB(1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_16DPSK_AsymDBPSK_End'}
   %%% Data[DBPSK,16DPSK] Train[AsymDBPSK]
   if (n == DB(1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK[2]_16DPSK_AsymDQPSK_End'}
   %%% Data[DQPSK,16DPSK] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK[2]_16DPSK_AsymDBPSK_End'}
   %%% Data[DBPSK,16DPSK] Train[AsymDBPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16DPSK');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'16QAM'}
   %%% Train[16QAM] Data[16QAM]
   v = ones(4,1);
   h = ones(1,4);
   rl = [-3*v, -1*v, 1*v, 3*v];
   im = [3*h; 1*h; -1*h; -3*h];
   d_array = rl + im*j;
   d_array = reshape(transpose(d_array),1,16);
   d_array = d_array./sqrt(mean(abs(d_array).^2));
case {'DQPSK_16QAM_AsymDQPSK_End'}
   %%% Data[DQPSK,16QAM] Train[AsymDQPSK]
   if (n == DB(1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_16QAM_AsymDBPSK_End'}
   %%% Data[DBPSK,16QAM] Train[AsymDBPSK]
   if (n == DB(1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK[2]_16QAM_AsymDQPSK_End'}
   %%% Data[DQPSK[2],16QAM] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK[2]_16QAM_AsymDBPSK_End'}
   %%% Data[DBPSK[2],16QAM] Train[AsymDBPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('16QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'64QAM'}
   %%% Train[64QAM] Data[64QAM]
   v = ones(8,1);
   h = ones(1,8);
   rl = [-7*v, -5*v, -3*v, -1*v, 1*v, 3*v, 5*v, 7*v];
   im = [7*h; 5*h; 3*h; 1*h; -1*h; -3*h; -5*h; -7*h];
   d_array = rl + im*j;
   d_array = reshape(transpose(d_array),1,64);
   d_array = d_array./sqrt(mean(abs(d_array).^2));
case {'DQPSK_64QAM_AsymDQPSK_End'}
   %%% Data[DQPSK,64QAM] Train[AsymDQPSK]
   if (n == DB(1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK_64QAM_AsymDBPSK_End'}
   %%% Data[DBPSK,64QAM] Train[AsymDBPSK]
   if (n == DB(1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+1 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DQPSK[2]_64QAM_AsymDQPSK_End'}
   %%% Data[DQPSK[2],64QAM] Train[AsymDQPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DQPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDQPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
case {'DBPSK[2]_64QAM_AsymDBPSK_End'}
   %%% Data[DBPSK[2],64QAM] Train[AsymDBPSK]
   if ((DB(1) <= n) & (n <= DB(1)+1))
      d_array = signal_constellation('DBPSK');
   elseif ((DB(1)+2 <= n) & (n <= DB(2)))
      d_array = signal_constellation('64QAM');
   elseif ((TB(1) <= n) & (n <= TB(2)))
      d_array = signal_constellation('AsymDBPSK');
   else
      error('*** SIGNAL_CONSTELLATION : Invalid n! ***');
   end;
% 'WQPSK_Z120','WQPSK_16QAM','WQPSK_16QPSK','WQPSK_C','WQPSK_Boost','QPSK_Boost'
% 'WDQPSK_Z120','WDQPSK_16QAM','WDQPSK_16QPSK','WDQPSK_C','WDQPSK_Boost','DQPSK_Boost'
% 'WQPSK_Boost2'
% 'WDQPSK_Boost2'
% 'WQPSK_Alt'
% 'WDQPSK_Alt'
% 'WDQPSK_DQPSK_64MRDPSK'
case {'WQPSK_Z120','WDQPSK_Z120'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = [0, 1, exp(120/360*2*pi*j), exp(240/360*2*pi*j)];
      sf = mean(abs(d_array).^2);
      d_array = d_array./sqrt(sf);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_16QAM','WDQPSK_16QAM'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = [+1+j, -3+j, -3-3*j, +1-3*j];
      sf = mean(abs(d_array).^2);
      d_array = d_array./sqrt(sf);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_16QPSK','WDQPSK_16QPSK'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = [+j, 2*exp(225/360*2*pi*j), 3, 4*exp(135/360*2*pi*j)];
      %d_array = [1, 2, 3, 4];
      sf = mean(abs(d_array).^2);
      d_array = d_array./sqrt(sf);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_C','WDQPSK_C'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = exp([0, 3, 6, 9]/16*2*pi*j);
      cen_d = mean(d_array);
      d_array = d_array - cen_d;
      sf = mean(abs(d_array).^2);
      d_array = d_array./sqrt(sf);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_Boost','WDQPSK_Boost'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = exp([0, 3, 6, 9]/16*2*pi*j);
      d_min = abs(d_array(1) - d_array(2));
      d_array = d_array/d_min*sqrt(8);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_Boost2','WDQPSK_Boost2'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = exp([0, 3, 6, 9]/16*2*pi*j);
      d_min = abs(d_array(1) - d_array(2));
      %d_array = d_array/d_min*sqrt(8);
      d_array = d_array * 10;
   elseif (n == 2)
      d_array = [+1, +j, -1, -j];
      d_array = d_array * 10;
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'QPSK_Boost','DQPSK_Boost'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = [+1, +j, -1, -j];
      d_min = abs(d_array(1) - d_array(2));
      d_array = d_array/d_min*sqrt(8);
   else
      d_array = [+1, +j, -1, -j];
   end;
case {'WQPSK_Alt','WDQPSK_Alt'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if ((n/2) == ceil(n/2))
      % Even
      d_array = [+1, +j, -1, -j];
   else
      % Odd
      d_array = exp([0, 3, 6, 9]/16*2*pi*j);
   end;
case {'WDQPSK_DQPSK_64MRDPSK'}
   %%% WARNING!!!
   error('*** SIGNAL_CONSTELLATION.M : Not Implemented Yet! ***');
   if (n == 1)
      d_array = exp([0, 3, 6, 9]/16*2*pi*j);
   elseif (n == 2)
      d_array = [+1, +j, -1, -j];
   else
      delta = 0.49;
      delta_0 = 0.5148^2;
      xi = sqrt(delta_0+([0:3]*delta));
      dxi = 2*pi/16;
      d_array = xi;
      for i = 1:15;
         d_array = [d_array, xi*exp(j*dxi*i)];
      end;
   end;
otherwise
   error('*** SIGNAL_CONSTELLATION.M : Unknown Data Type! ***');
end;